function Ascatt = A_SDH_Psep(setup)
% A_SDH_PSEP computes the separation of variables solution
% for the 3-D non-dimensional pulse-echo P-wave 
% scattering amplitude, Ascatt, for a side-drilled hole 
% of radius b (in mm).
% The function returns the scattering amplitude, A, divided
% by the length, L, i.e. Ascatt = A/L so that a value for L
% does not need to be specified.The only input to the function
% is the setup structure. The calling sequence is:
% Ascatt =A_SDH_Psep(setup);

% get setup parameters
f =setup.f;
b = setup.flaw.b;
cp =setup.matl.cp2;
cs = setup.matl.cs2;
%
cr = cp/cs;                 % ratio of P- and S- wave speeds
kp = 2000*pi*b*f./cp;      % non-dimensional wave number, P-waves
kp = kp + .0001*(kp == 0); 

% break P-wave wave number into two regions: kp < 2 and kp >= 2
indc = find(kp < 2.);      
kpd =kp(indc);
ind2 =find(kp >= 2.);
kpu = kp(ind2);
% S-wave wave numbers over same ranges
ksd =cr*kpd;
ksu =cr*kpu;

% use relatively small, fixed number of terms for kp <2
num = 10;
% compute normalized scattering amplitude over kp <2 for sphere of radius b
A1 =sca(kpd, ksd, num);

% use much larger number of terms for kp >= 2
num2= 10 + round(kpu(end));
% compute normalized scattering amplitude over kp >= 2 for sphere of radius b
A2 = sca(kpu,ksu, num2);

% combine two ranges
Ascatt= [A1  A2];
% force zero frequency normalized scattering amplitude to zero exactly
Ascatt(1) =0;

% subfunction for calculating normalized scattering amplitude for a 
% side- drilled hole with a given number of terms in the series. 
% Generally, ten terms should be adequate
% for kp < 2 and a number of terms that is proportional to the max 
% kp-value should be adequate for large kp values. However, if the
% max kp-value is very large, the number of terms used here based on this
% value may be too large for the values just above kp =2, resulting in the
% round-offs that cause the function to return NaNs at those lower
% frequencies. This function has been tested up to kp = 90 without problems
% of this sort.
% This function uses Hankel functions of type m, order n defined by the MATLAB 
% function besselh(n, m, x)

function A = sca(kp,ks, numb)
% initialize arrays
An = zeros(size(kp));      
Ckp1 =zeros(size(kp));
Ckp2 =zeros(size(kp));
Cks1 =zeros(size(kp));
Dkp1 =zeros(size(kp));
Dkp2 =zeros(size(kp));
Dks1 =zeros(size(kp));
c =zeros(size(kp));

%calculate the series
for n = 0:numb 

    Ckp1 =(n^2 +n -(ks.^2/2)).*besselh(n, 1,kp) -((2*n).*besselh(n,1,kp) -kp.*besselh(n+1,1,kp));
    Ckp2 =(n^2 +n -(ks.^2/2)).*besselh(n, 2,kp) -((2*n).*besselh(n,2,kp) -kp.*besselh(n+1,2,kp));
    Cks1 =(n^2 +n -(ks.^2/2)).*besselh(n, 1,ks) -((2*n).*besselh(n,1,ks) -ks.*besselh(n+1,1,ks));
    Dkp1 = (n^2 +n).*besselh(n,1,kp) -n*((2*n).*besselh(n,1,kp) -kp.*besselh(n+1,1,kp));
    Dkp2 = (n^2 +n).*besselh(n,2,kp) -n*((2*n).*besselh(n,2,kp) -kp.*besselh(n+1,2,kp));
    Dks1 = (n^2 +n).*besselh(n,1,ks) -n*((2*n).*besselh(n,1,ks) -ks.*besselh(n+1,1,ks));

if n == 0
    c = 1+ Ckp2./Ckp1;
else
c = 2*(1+(Ckp2.*Cks1 -Dkp2.*Dks1)./(Ckp1.*Cks1 - Dkp1.*Dks1));
end
An = An + ((-1.)^n)*c; 
end 
% Now, put the external factor in
A = (i/(2*pi))*An;
