function Ascatt = A_SDH_Ssep(setup)
% A_SDH_SSEP computes the separation of variables solution
% for the 3-D non-dimensional pulse-echo SV-wave 
% scattering amplitude, Ascatt, for a side-drilled hole 
% of radius b (in mm).
% The function returns the scattering amplitude, A, divided
% by the length, L, i.e. Ascatt = A/L so that a value for L
% does not need to be specified.The only input to the function
% is the setup structure. The calling sequence is:
% Ascatt =A_SDH_Ssep(setup);
% 

% get input parameters
f=setup.f;
b =setup.flaw.b;
cp = setup.matl.cp2;
cs =setup.matl.cs2;

cr = cp/cs;                 % ratio of P- and S- wave speeds
ks = 2000*pi*b*f./cs;      % non-dimensional wave number, S-waves
ks = ks + .001*(ks == 0); 

% break S-wave wave number into two regions: ks < 5 and ks >= 5
indc = find(ks < 5);      
ksd =ks(indc);
ind2 =find(ks >= 5);
ksu = ks(ind2);
% P-wave wave numbers over same ranges
kpd =ksd./cr;
kpu =ksu./cr;

% use relatively small, fixed number of terms for ks <5
num = 10;
% compute scattering amplitude over ks < 5 for sphere of radius b
A1 =sca(kpd, ksd, num);

% use much larger number of terms for ks >= 5
num2= 10 + round(ksu(end));
% compute scattering amplitude over ks >= 5 for sphere of radius b
A2 = sca(kpu,ksu, num2);

% combine two ranges
Ascatt= [A1  A2];
% force zero frequency scattering amplitude to zero exactly
Ascatt(1) =0;

% subfunction for calculating normalized scattering amplitude for a 
% side-drilled hole with a given number of terms in the series. 
% Generally, ten terms should be adequate
% for ks < 5 and a number of terms that is proportional to the max 
% ks-value should be adequate for large ks values. However, if the
% max ks-value is very large, the number of terms used here based on this
% value may be too large for the values just above ks = 5, resulting in the
% round-offs that cause the function to return NaNs at those lower
% frequencies. This function has been tested up to ks = 50 without problems
% of this sort.
% This function uses Hankel functions of type m, order n defined by the MATLAB 
% function besselh(n, m, x)


function A = sca(kp, ks, numb)

% initialize arrays
An = zeros(size(kp));      
Cnp1 =zeros(size(kp));
Cns1 =zeros(size(kp));
Cns2 =zeros(size(kp));
Dnp1 =zeros(size(kp));
Dnp2 =zeros(size(kp));
Dns1 =zeros(size(kp));
c =zeros(size(kp));

% Calculate series
for n = 0:numb 

    Cnp1 =(n^2 +n -(ks.^2/2)).*besselh(n, 1,kp) -((2*n).*besselh(n,1,kp) -kp.*besselh(n+1,1,kp));
    Cns2 =(n^2 +n -(ks.^2/2)).*besselh(n, 2,ks) -((2*n).*besselh(n,2,ks) -ks.*besselh(n+1,2,ks));
    Cns1 =(n^2 +n -(ks.^2/2)).*besselh(n, 1,ks) -((2*n).*besselh(n,1,ks) -ks.*besselh(n+1,1,ks));
    Dnp1 = (n^2 +n).*besselh(n,1,kp) -n*((2*n).*besselh(n,1,kp) -kp.*besselh(n+1,1,kp));
    Dns2 = (n^2 +n).*besselh(n,2,ks) -n*((2*n).*besselh(n,2,ks) -ks.*besselh(n+1,2,ks));
    Dns1 = (n^2 +n).*besselh(n,1,ks) -n*((2*n).*besselh(n,1,ks) -ks.*besselh(n+1,1,ks));

if n == 0
    c = 1+ Cns2./Cns1;
else
c = 2*(1+(Cns2.*Cnp1 -Dns2.*Dnp1)./(Cnp1.*Cns1 - Dnp1.*Dns1));
end
An = An + ((-1.)^n)*c; 
end 
% Now, put the external factor in
A = (i/(2*pi))*An;
