function Aout = A_void_Psep(setup) 
% A_VOID_PSEP  computes the far field P-wave scattering amplitude, Aout ,
% for a spherical void of radius b in an elastic solid (pulse echo) 
% using the method of separation of variables. the only input parameter is
% the setup structure. The complex scattering amplitude, Aout,
% is returned (in mm). 
% The calling sequence is: 
% Aout = A_void_Psep(setup)

% get input parameters
f=setup.f;
b =setup.flaw.b;
cp = setup.matl.cp2;
cs =setup.matl.cs2;

cr = cp/cs;                 % ratio of P- and S- wave speeds
kp = 2000*pi*b*f./cp;      % non-dimensional wave number, P-waves
kp = kp + .0001*(kp == 0); 

% break P-wave wave number into two regions: kp < 2 and kp >= 2
indc = find(kp < 2.);      
kpd =kp(indc);
ind2 =find(kp >= 2.);
kpu = kp(ind2);
% S-wave wave numbers over same ranges
ksd =cr*kpd;
ksu =cr*kpu;

% use relatively small, fixed number of terms for kp <2
num = 10;
% compute scattering amplitude over kp <2 for sphere of radius b
A1 =sca(kpd, ksd, num, b);

% use much larger number of terms for kp >= 2
num2= 10 + round(kpu(end));
% compute scattering amplitude over kp >= 2 for sphere of radius b
A2 = sca(kpu,ksu, num2, b);

% combine two ranges
Aout= [A1  A2];
% force zero frequency scattering amplitude to zero exactly
Aout(1) =0;

% subfunction for calculating scattering amplitude with a given number
% of terms in the series. Generally, ten terms should be adequate
% for kp < 2 and a number of terms that is proportional to the max 
% kp-value should be adequate for large kp values. However, if the
% max kp-value is very large, the number of terms used here based on this
% value may be too large for the values just above kp =2, resulting in the
% round-offs that cause the function to return NaNs at those lower
% frequencies. This function has been tested up to kp = 90 without problems
% of this sort.

function A = sca(xp,xs, numb, b)
An = zeros(size(xp));       % initialize array of zeros

% First compute the normalized scattering amplitude A/b.

% xp = P- wave number, xs = S- wave number, k is an integer.
% Uses spherical Bessel functions and spherical Hankel functions
% of order k defined by sphJ(k,x), sphH(k, x)

for k = 0:numb 
e3 = (2.*k+1).*((k.^2 - k - xs.^2./2).*sphJ(k, xp) +2.*xp.*sphJ(k+1,xp));
e4 = (2.*k+1).*((k-1).*sphJ(k, xp) - xp.*sphJ(k+1, xp));
e32 = -k.*(k+1).*((k-1).*sphH(k, xs) - xs.*sphH(k+1, xs));
e31 = (k.^2 - k - xs.^2./2).*sphH(k, xp) + 2.*xp.*sphH(k+1, xp);
e41 = (k - 1).*sphH(k, xp) -xp.*sphH(k+1, xp);
e42 = -(k.^2 -1 - xs.^2./2).*sphH(k, xs) - xs.*sphH(k+1, xs);
if k == 0
    c = e3./e31;
else
c = (e3.*e42 - e4.*e32)./(e31.*e42 - e41.*e32);
end
An = An + ((-1.)^k)*c; 
end 

% Now, put the b factor back, insert i/kp term which multiplies entire result
A = i*b*An./xp;