function Aout = A_void_Ssep(setup) 
% A_VOID_SSEP  computes the far field SV-wave scattering amplitude, Aout ,
% for a spherical void of radius b in an elastic solid (pulse echo) 
% using the method of separation of variables. the only input parameter is
% the setup structure. The complex scattering amplitude, Aout,
% is returned (in mm). 
% The calling sequence is: 
% Aout = A_void_Ssep(setup)

% get input parameters
f=setup.f;
b =setup.flaw.b;
cp = setup.matl.cp2;
cs =setup.matl.cs2;

cr = cp/cs;                 % ratio of P- and S- wave speeds
ks = 2000*pi*b*f./cs;      % non-dimensional wave number, S-waves
ks = ks + .001*(ks == 0); 

% break S-wave wave number into two regions: ks < 5 and ks >= 5
indc = find(ks < 5);      
ksd =ks(indc);
ind2 =find(ks >= 5);
ksu = ks(ind2);
% P-wave wave numbers over same ranges
kpd =ksd./cr;
kpu =ksu./cr;

% use relatively small, fixed number of terms for ks <5
num = 10;
% compute scattering amplitude over ks < 5 for sphere of radius b
A1 =sca(kpd, ksd, num, b);

% use much larger number of terms for ks >= 5
num2= 10 + round(ksu(end));
% compute scattering amplitude over ks >= 5 for sphere of radius b
A2 = sca(kpu,ksu, num2, b);

% combine two ranges
Aout= [A1  A2];
% force zero frequency scattering amplitude to zero exactly
Aout(1) =0;

% subfunction for calculating scattering amplitude with a given number
% of terms in the series. Generally, ten terms should be adequate
% for ks < 5 and a number of terms that is proportional to the max 
% ks-value should be adequate for large ks values. However, if the
% max ks-value is very large, the number of terms used here based on this
% value may be too large for the values just above ks=5, resulting in the
% round-offs that cause the function to return NaNs at those lower
% frequencies. this function has been tested up to ks = 50 without problems
% of this sort.

function A = sca(xp,xs, numb, b)
An = zeros(size(xp));       % initialize array of zeros

% First compute the normalized scattering amplitude A/b.

% xp = P- wave number, xs = S- wave number, k is an integer.
% Uses spherical Bessel functions and spherical Hankel functions
% of order k defined by sphJ(k,x), sphH(k, x)

for k = 1:numb 
j12 = k.*(k+1).*((k-1).*sphJ(k, xs)-xs.*sphJ(k+1,xs));
h12 = k.*(k+1).*((k-1).*sphH(k, xs) -xs.*sphH(k+1,xs));
h13 =((k.^2-k-xs.^2./2).*sphH(k,xp) +2.*xp.*sphH(k+1, xp));
j41 =((k-1).*sphJ(k, xs) -xs.*sphJ(k+1, xs))./2;
h41 = ((k-1).*sphH(k, xs) -xs.*sphH(k+1, xs))./2;
j42 = (k.^2 -1 -xs.^2./2).*sphJ(k, xs) + xs.*sphJ(k+1, xs);
h42 =((k.^2 -1 -xs.^2./2).*sphH(k, xs) + xs.*sphH(k+1, xs));
h43 = ((k-1).*sphH(k, xp) - xp.*sphH(k+1, xp));

c = (h13.*j42 -j12.*h43 )./(h13.*h42 -h12.*h43) -j41./h41;

An = An + (-1)^k.*((2.*k+1)./2).*c./(-i.*xs); 
end 

% Now, put the b factor back
A = b*An;