function s = exp_systf(setup)
% EXP_SYSTF generates the system function from the measured 
% voltage received by a circular, planar or focused immersion
% transducer due to the waves reflected from the planar front 
% surface of a solid. It is assumed that the solid is the same as the one
% present in a flaw measurement where this system function is to be used.
% The fluid path length here can be different from the one used in the  
% flaw measurement. 
% 
% This function assumes that the MAT-file that is loaded includes
% a row vector ref that contains the sampled voltage values
% measured from the front surface of the solid in a reference experiment
% and a row vector t_ref that contains the corresponding sampled time 
% values (whose actual values are not needed here, only the sampling time
% interval, dt, between points).
%
% The calling sequence is:
% s = exp_systf(setup);

filename =setup.system.ref_file;
load(filename) % load reference wave form (contained in the row vector ref)
% and the time axis values (contained in the row vector t_ref) from a MAT-file

dt = t_ref(2)-t_ref(1);
% calculate Fourier Transform
V =FourierT(ref, dt);
%generate frequency axis
[nr,nc] = size(ref);
fs = s_space(0, 1/dt, nc);


% get setup frequency values and check for consistency
f = setup.f;
df= f(2) - f(1);
dfs =fs(2) - fs(1);
fsize=size(f);
numf = fsize(2);
if df > (dfs + .001) | df < (dfs - .001)
    error('frequency spacing mismatch of setup and exp values')
end
if f(end) > (fs(end) +dfs)/2
    error('max frequency in setup exceeds Nyquist')
end
% keep number of measured voltage frequency components
% compatible with that in setup
Vc=V(1:numf); 
% get remaining setup parameters needed
z1r =setup.system.z1r;
en =setup.system.en;
d1 =setup.matl.d1;
cp1 = setup.matl.cp1;
d2 = setup.matl.d2;
cp2 = setup.matl.cp2;
cs2 = setup.matl.cs2;
a = setup.trans.d/2;
p1 =setup.matl.p1;
alphac =p1(3); % frequency squared attenuation coefficient for water
fl = setup.trans.fl;

% if transducer is focused, z1r must be the same as the focal length
if fl ~= inf
    if z1r > fl +.01  | z1r < fl - .01
        warning(' reference water path specified is not focal length, using focal length')
        z1r = fl;
    end
end

% calculate wave number , reflection coefficient of fluid-solid interface
% and argument for acoustic/elastic transfer function
ka =2000.*pi.*f.*a./cp1;
R12 = (cp2*d2 - cp1*d1)/(cp2*d2 + cp1*d1);
arg = (a/z1r)*ka;
alpha = alphac*f.^2;
% calculate acoustic-elastic transfer function, leave out propagation phase

ta = 2*R12*exp(-2*alpha.*z1r).*(1 -exp(i*arg/2).*(BesselJ(0, arg/2) -i*BesselJ(1, arg/2)));
if fl ~= inf
    ta = -conj(ta);
end

% deconvolve measured voltage frequency components with transfer function
% to get system function
s = Wiener_filter(Vc, ta, en);